﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Ozeki.Media;
using Ozeki.Media.MediaHandlers;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.Media;
using Ozeki.VoIP.SDK;
using Ozeki.Common;

namespace VoiceCallRecording
{
    public partial class Softphone : Form
    {

        public bool IsRegRequired = true;
        public String displayname;
        public String username;
        public String registername;
        public String regpass;
        public String domainhost;
        public int port;
        public NatTraversalMethod natTraversal = 0;

        ISoftPhone softPhone;
        IPhoneLine phoneLine;
        PhoneLineState phoneLineInformation;
        IPhoneCall call;
        Microphone microphone = Microphone.GetDefaultDevice();
        Speaker speaker = Speaker.GetDefaultDevice();
        MediaConnector connector = new MediaConnector();
        PhoneCallAudioSender mediaSender = new PhoneCallAudioSender();
        PhoneCallAudioReceiver mediaReceiver = new PhoneCallAudioReceiver();
        bool inComingCall;

        //recording support tools
        WaveStreamRecorder recorder;
        AudioMixerMediaHandler mixer = new AudioMixerMediaHandler();

        string filename;
        string caller;

        public Softphone()
        {
            InitializeComponent();
            //some devices are missing
            string message = String.Empty;

            if (microphone == null)
                message += "You have no microphone attached to your computer, please note that your partner will not hear your voice.\n";
            if (speaker == null)
                message += "You have no speaker attached to your computer, please note that you will not hear your partner.\n";
            
            if (message != String.Empty)
                MessageBox.Show(message, "Some devices are missing", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void OKButton_Click(object sender, EventArgs e)
        {
            displayname = DisplayNameTB.Text;
            username = UserNameTB.Text;
            registername = RegNameTB.Text;
            regpass = RegPwdTB.Text;

            try
            {
                int i;
                i = Int32.Parse(IP1.Text);
                if (!(i >= 0 && i <= 255))
                    throw new Exception();
                i = Int32.Parse(IP2.Text);
                if (!(i >= 0 && i <= 255))
                    throw new Exception();
                i = Int32.Parse(IP3.Text);
                if (!(i >= 0 && i <= 255))
                    throw new Exception();
                i = Int32.Parse(IP4.Text);
                if (!(i >= 0 && i <= 255))
                    throw new Exception();
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("The IP address you set is invalid, please set a valid IP address.\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
            MessageBoxIcon.Error);
            }
            domainhost = string.Empty;
            domainhost = IP1.Text + "." + IP2.Text + "." + IP3.Text + "." + IP4.Text;
            try
            {
                port = Int32.Parse(PortNo.Text);
            }
            catch (Exception ex)
            {

                MessageBox.Show(String.Format("The port number you set is invalid, please set a valid port number.\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
            MessageBoxIcon.Error);
            }
            switch (NAT.SelectedIndex)
            {
                case -1:
                case 0:
                    natTraversal = NatTraversalMethod.None;
                    break;
                case 1:
                    natTraversal = NatTraversalMethod.STUN;
                    break;
                case 2:
                    natTraversal = NatTraversalMethod.TURN;
                    break;
            }
            InitializeSoftPhone();
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            IsRegRequired = checkBox1.Checked;
        }

        private void PickUp_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Please note, that all your calls made by this softphone will be recorded.", "", MessageBoxButtons.OK, MessageBoxIcon.Information);
            if (inComingCall)
            {
                inComingCall = false;
                call.Accept();
                return;
            }

            if (call != null)
                return;

            if (string.IsNullOrEmpty(DialedNumber.Text))
                return;

            if (phoneLineInformation != PhoneLineState.RegistrationSucceeded && phoneLineInformation != PhoneLineState.NoRegNeeded)
            {
                MessageBox.Show("Phone line state is not valid!");
                return;
            }
            caller = DialedNumber.Text;
            call = softPhone.CreateCallObject(phoneLine, DialedNumber.Text);
            WireUpCallEvents();
            call.Start();
        }

        private void HangUp_Click(object sender, EventArgs e)
        {
            if (call != null)
            {
                if (inComingCall && call.CallState == CallState.Ringing)
                    call.Reject();
                else
                    call.HangUp();
                inComingCall = false;
                call = null;
            }
            DialedNumber.Text = string.Empty;
        }

        private void KeyPadButton_Click(object sender, EventArgs e)
        {
            var btn = sender as Button;
            if (btn != null)
            {
                if (btn.Text == "0/+")
                {
                    DialedNumber.Text += "0";
                }
                else
                    DialedNumber.Text += btn.Text.Trim();
            }
        }

        private void InitializeSoftPhone()
        {
            try
            {
                softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5700, 5750, 5700);
                softPhone.IncomingCall += new EventHandler<VoIPEventArgs<IPhoneCall>>(softPhone_IncomingCall);

                SIPAccount sa = new SIPAccount(IsRegRequired, displayname, username, registername, regpass, domainhost, port);
                NatConfiguration nc = new NatConfiguration(natTraversal);

                phoneLine = softPhone.CreatePhoneLine(sa, nc);
                phoneLine.PhoneLineStateChanged += new EventHandler<VoIPEventArgs<PhoneLineState>>(phoneLine_PhoneLineInformation);

                softPhone.RegisterPhoneLine(phoneLine);
                DialedNumber.Text = string.Empty;
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("You didn't set the right IP address. Please set your IP address and try again\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
                MessageBoxIcon.Error);
            }
        }

        private void InvokeGUIThread(Action action)
        {
            Invoke(action);
        }

        private void softPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            InvokeGUIThread(() =>
            {
                RegistrationState.Text = "Incoming call";
                DialedNumber.Text = String.Format("from {0}", e.Item.DialInfo);
                caller = e.Item.DialInfo.DisplayName;
                call = e.Item;
                WireUpCallEvents();
                inComingCall = true;
            });
        }

        private void WireUpCallEvents()
        {
            call.CallStateChanged += new EventHandler<VoIPEventArgs<CallState>>(call_CallStateChanged);
            call.CallErrorOccured += new EventHandler<VoIPEventArgs<CallError>>(call_CallErrorOccured);
        }

        private void WireDownCallEvents()
        {
            if (call != null)
            {
                call.CallStateChanged -= (call_CallStateChanged); ;
                call.CallErrorOccured -= (call_CallErrorOccured);
            }
        }

        private void call_CallErrorOccured(object sender, VoIPEventArgs<CallError> e)
        {
            InvokeGUIThread(() =>
            {
                RegistrationState.Text = e.Item.ToString();
            });
        }

        private void call_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            InvokeGUIThread(() => { RegistrationState.Text = e.Item.ToString(); });

            switch (e.Item)
            {
                case CallState.InCall:
                    if (microphone != null)
                        microphone.Start();
                    connector.Connect(microphone, mediaSender);
                    
                    filename = caller + "-" + DateTime.Now.Hour.ToString()+"-"+ DateTime.Now.Minute.ToString()+"-"+DateTime.Now.Second.ToString()+".wav";
                    recorder = new WaveStreamRecorder(filename);

                    connector.Connect(microphone, mixer);
                    connector.Connect(mediaReceiver, mixer);
                    connector.Connect(mixer, recorder);
                    if (speaker != null)
                        speaker.Start();
                    connector.Connect(mediaReceiver, speaker);

                    mediaSender.AttachToCall(call);
                    mediaReceiver.AttachToCall(call);

                    recorder.StartStreaming();
                    recorder.IsStreaming = true;

                    break;
                case CallState.Completed:
                    recorder.StopStreaming();
                    recorder.IsStreaming = false;

                    connector.Disconnect(microphone, mixer);
                    connector.Disconnect(mediaReceiver, mixer);
                    connector.Disconnect(mixer, recorder);
                    if (microphone != null)
                        microphone.Stop();
                    connector.Disconnect(microphone, mediaSender);
                    if (speaker != null)
                        speaker.Stop();
                    connector.Disconnect(mediaReceiver, speaker);

                    mediaSender.Detach();
                    mediaReceiver.Detach();

                    WireDownCallEvents();
                    call = null;

                    InvokeGUIThread(() => { DialedNumber.Text = string.Empty; });
                    break;
                case CallState.Cancelled:
                    WireDownCallEvents();
                    call = null;
                    break;
            }
        }

        private void phoneLine_PhoneLineInformation(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            phoneLineInformation = e.Item;
            InvokeGUIThread(() =>
            {
                Username.Text = ((IPhoneLine)sender).SIPAccount.RegisterName;
                if (e.Item == PhoneLineState.RegistrationSucceeded)
                {
                    ConnectionState.Text = "Online";
                    RegistrationState.Text = "Registration succeeded";
                }
                else
                    RegistrationState.Text = e.Item.ToString();
            });
        }
        
        void WaveStreamRecorder_Stopped(object sender, EventArgs e)
        {
            if (RecorderStopped != null)
                RecorderStopped(this, EventArgs.Empty);
        }

        public event EventHandler<EventArgs> RecorderStopped;

        private void Softphone_Load(object sender, EventArgs e)
        {
            
        }
    }
}
